{                             NOMBRE NUMEROLOGIQUE

Auteur : Bacterius.
Le code ci-dessous est utilisable dans n'importe-quel projet non terroriste.
Cette unit calcule le nombre numrologique d'une chane quelconque. Ce code
est 100% compatible avec tous les systmes d'exploitations/langages/chanes.
Sur des systmes autres que Windows XP et postrieur, il faudra peut-tre
changer le tableau d'quivalence en fonction des codes ASCII de votre systme.
Les routines sont trs rapides, la premire sous-routine a pour complexit O(l)
o l est la longueur de la chane, et le nombre intermdiaire ne peut excder
9l. La seconde sous-routine a pour complexit O(log(l)), car chaque somme
intermdiaire ne peut excder 9n o n est la longueur de la somme intermdiaire,
ce qui a pour effet de rduire la taille de la somme de n  plus ou moins log(n)
 chaque itration. La complexit totale de l'algorithme est donc de O(l log(l))
et l'algorithme est donc trs rapide. La premire sous-routine demande une
addition et une lecture de tableau par caractre de la chane, et la seconde
demande une poigne (en gnral, pas plus de deux ou trois) divisions et modulos
par une puissance de 10. Des commentaires sont inclus. Bonne programmation !   }

unit UCalculs;

interface

{ Cette fonction renvoie le nombre numrologique d'une chane quelconque }
function ComputeNumerologicNumber(const Str: String): Longword;

implementation

const
  { Ici, chaque caractre ASCII est associ  une valeur numrologique comprise entre 1 et 9.
    - les minuscules sont interprtes comme des majuscules (ou vice versa :p)
    - les accents sont intrprts comme la lettre sur laquelle ils reposent
    - tous les autres caractres sont ignors, donc zro }

  EquivalenceArray: array [Byte] of Byte = (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                            0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 1, 2, 3, 4, 5, 6,
                                            7, 8, 9, 1, 2, 3, 4, 5, 6, 7, 8, 0, 0, 0, 0, 0,
                                            0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 1, 2, 3, 4, 5, 6,
                                            7, 8, 9, 1, 2, 3, 4, 5, 6, 7, 8, 0, 0, 0, 0, 0,
                                            0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
                                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 7,
                                            0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                            1, 1, 1, 1, 1, 1, 0, 3, 5, 5, 5, 5, 9, 9, 9, 9,
                                            4, 5, 6, 6, 6, 6, 6, 0, 0, 3, 3, 3, 3, 7, 0, 0,
                                            1, 1, 1, 1, 1, 1, 0, 3, 5, 5, 5, 5, 9, 9, 9, 9,
                                            0, 5, 6, 6, 6, 6, 6, 0, 0, 3, 3, 3, 3, 7, 0, 7);

{ Cette fonction rcupre la somme simple des lettres dans la chane. }
function PrepareString(const Str: String): Longword;
Var
 P: PByte;
 E: Pointer;
begin
 { On prpare les pointeurs }
 P := @PAnsiChar(Str)^;
 E := Ptr(Longword(P) + Longword(Length(Str)));
 Result := 0;

 { On parcourt la chane }
 while P <> E do
  begin
   { On incrmente le rsultat selon le tableau EquivalenceArray }
   Inc(Result, EquivalenceArray[P^]);

   { On passe au caractre suivant }
   Inc(P);
  end;
end;

{ Cette fonction va itrativement rduire un nombre en additionnant ses chiffres. }
function ReduceSum(Sum: Longword): Longword;
begin
 { Tant que la somme a plus d'un chiffre }
 while Sum > 9 do
  begin
   { On initialise la somme intermdiaire }
   Result := 0;

   { On prend chaque chiffre individuellement, et on l'ajoute au rsultat }
   { On prend le premier chiffre }
   Inc(Result, Sum mod 10);
   { On prend le deuxime chiffre }
   Inc(Result, (Sum div 10) mod 10);
   { Si la somme fait trois chiffres, on prend le troisime chiffre }
   if Sum > 99 then Inc(Result, (Sum div 100) mod 10);
   { Ainsi de suite ... }
   if Sum > 999 then Inc(Result, (Sum div 1000) mod 10);
   { C'est improbable qu'une somme fasse plus de dix mille, mme pour des trs grandes chanes, donc ...
     Mais si vous utilisez d'normes chanes (plus de 1111 caractres utiles), rajoutez des lignes ! }

   { On met  jour la prochaine somme }
   Sum := Result;
  end;

 { Le rsultat est la somme restante }
 Result := Sum;
end;

{ Cette fonction renvoie le nombre numrologique d'une chane quelconque }
function ComputeNumerologicNumber(const Str: String): Longword;
begin
 { On se sert de nos deux prcdentes fonctions, c'est trs simple maintenant }
 Result := ReduceSum(PrepareString(Str));
end;

end.
